require 'spec_helper'

describe BeanstalkFarmer::Job do
  let(:name) { 'bacon' }

  let(:job) {
    double('job', body: "[\"#{name}\",{\"msg\":\"Hello\"}]", name: name, args: { 'msg' => 'Hello' }, ttr: 56, delete: true)
  }

  let(:delete_handler) { lambda { |args| [:delete] } }

  let(:release_handler) { lambda { |args| [:release, 3, 240] } }

  describe '.new' do
    it 'sets the job' do
      described_class.new(job).job.should == job
    end

    it 'decodes the name of the job' do
      described_class.new(job).name.should == name
    end

    it 'decodes the arguments for the job' do
      described_class.new(job).args.should == job.args
    end
  end

  describe '.handler_pool' do
    it 'is a Hash of job handlers' do
      described_class.handler_pool.should be_a_kind_of(Hash)
    end

    it 'can reset the Hash of job hanlders' do
      described_class.handler_pool['bacon'] = lambda { |args| args }
      described_class.handler_pool(true)
      described_class.handler_pool.should be_empty
    end
  end

  describe '#handler' do
    let(:processor) { double(:processor, call: true) }

    subject { described_class.new(job) }

    before do
      described_class.handler_pool[name] = processor
    end

    after do
      described_class.handler_pool(true)
    end

    it 'is the handler class used to process the job' do
      subject.handler.should == processor
    end
  end

  describe '#work' do
    let(:connection) { double('Connection', job_stats: { ttr: 3 }) }

    let(:name) { 'my_job' }

    let(:body) { "[\"#{name}\",{\"msg\":\"Hello\"}]" }

    after(:each) do
      described_class.handler_pool(:reset)
    end

    context '(general behaviour)' do
      let(:handler) { double('handler', call: true) }

      let(:job) { double('job', body: body, ttr: 3, delete: true) }

      subject { described_class.new(job) }

      before do
        described_class.handler_pool[name] = handler
      end

      it 'uses the handler to perform some work' do
        handler.should_receive(:call)
        subject.work
      end

      it 'times out if the handler cannot complete in time' do
        handler.should_receive(:call).and_raise(Timeout::Error)

        expect {
          subject.work
        }.to raise_error(BeanstalkFarmer::TimedOut)
      end

      it 'deletes the job by default' do
        job.should_receive(:delete).once
        subject.work
      end
    end

    context '(response handling with no arguments)' do
      let(:action) { :delete }

      let(:handler) { lambda { |args| [action] } }

      let(:job_id) { 3 }

      let(:job) { Beanstalk::Job.new(connection, job_id, body) }

      subject { described_class.new(job) }

      before do
        described_class.handler_pool[name] = handler
      end

      it 'calls the specified action' do
        connection.should_receive(:delete).with(job_id)
        subject.work
      end
    end

    context 'response handling with arguments' do
      let(:action) { :release }

      let(:new_priority) { 3 }

      let(:new_delay) { 240 }

      let(:handler) { lambda { |args| [action, new_priority, new_delay] } }

      let(:job_id) { 1 }

      let(:job) { Beanstalk::Job.new(connection, job_id, body) }

      subject { described_class.new(job) }

      before do
        described_class.handler_pool[name] = handler
      end

      it 'reserves the job with appropriate arguments' do
        connection.should_receive(action).with(job_id, new_priority, new_delay)
        subject.work
      end
    end

    context 'response handling with empty arguments' do
      let(:handler) { lambda { |args| Array.new } }

      let(:job_id) { 1 }

      let(:job) { Beanstalk::Job.new(connection, job_id, body) }

      subject { described_class.new(job) }

      before do
        described_class.handler_pool[name] = handler
      end

      it 'deletes the job when there are empty arguments' do
        connection.should_receive(:delete).with(job_id)
        subject.work
      end
    end
  end
end
