require 'spec_helper'

describe BeanstalkFarmer::Service, :beanstalk_required do
  subject { described_class.new }

  its(:host) { should == BeanstalkFarmer::Config.host }

  its(:port) { should == BeanstalkFarmer::Config.port }

  describe '#uri' do
    it 'is a formatted URI for a Beanstalk queue' do
      subject.uri.should =~ /\A#{subject.host}:#{subject.port}\z/
    end
  end

  describe '#connection' do
    context 'when successful' do
      after(:each) do
        subject.close
      end

      it 'establishes a connection to Beanstalk' do
        expect {
          subject.connection.use 'fake.tube'
        }.to_not raise_error(BeanstalkFarmer::NotConnectedError)
      end
    end

    context 'when failing' do
      it 'raises an exception if a connection cannot be established' do
        Beanstalk::Pool.stub(:new).and_raise(Beanstalk::NotConnected)
        expect {
          subject.connection
        }.to raise_error(BeanstalkFarmer::NotConnectedError)
      end
    end
  end

  describe '#close' do
    it 'closes the Beanstalk connection' do
      connection = subject.connection # ensure we're connected
      subject.close
      connection.open_connections.should be_empty
    end
  end

  describe '#watch_tubes' do
    after(:each) do
      subject.close
    end

    it 'adds a tube to the list of watched tubes' do
      subject.watch_tubes(%w[bacon])
      subject.connection.list_tubes_watched.should have_tube_named('bacon')
    end
  end

  describe '#ignore_unwatched_tubes' do
    after(:each) do
      subject.close
    end

    it 'ignores tubes that are not being watched' do
      pending 'a creative way to test this problem'
    end
  end

  describe '#reserve' do
    let(:job) {
      double('job', body: "[\"bacon\",{\"msg\":\"Hello\"}]", name: 'bacon', args: { 'msg' => 'Hello' }, ttr: 56)
    }

    before(:each) do
      subject.connection.stub(:reserve) { job }
    end

    after(:each) do
      subject.close
    end

    it 'builds a Farmer::Job' do
      subject.reserve.should be_a_kind_of(BeanstalkFarmer::Job)
    end

    it 'reserves a job indefinitely' do
      subject.connection.should_receive(:reserve).with(nil) { job }
      subject.reserve
    end

    it 'reserves a job for the specified number of seconds' do
      subject.connection.should_receive(:reserve).with(30) { job }
      subject.reserve(30)
    end

    it 'raises an exception if the reserved job runs out of time' do
      subject.connection.stub(:reserve).and_raise(Beanstalk::TimedOut)

      expect {
        subject.reserve(30)
      }.to raise_error(BeanstalkFarmer::TimedOut)
    end
  end

  describe '#reserve_and_work_job' do
    let(:job) { double(work: true) }

    before(:each) do
      subject.stub(:reserve) { job }
    end

    it 'reserves a job' do
      subject.should_receive(:reserve) { job }
      subject.reserve_and_work_job
    end

    it 'passes an optional timeout to the reserved job' do
      subject.should_receive(:reserve).with(30) { job }
      subject.reserve_and_work_job(30)
    end

    it 'works the reserved job' do
      job.should_receive(:work)
      subject.reserve_and_work_job
    end
  end
end
